#include "riff.h"
#include "common.h"

namespace harkio {
using namespace std;

/**********************************************************************
 RIFF基底クラスメンバ関数
 **********************************************************************/

// サイズフィールドを確定する。
// (データ構造を再帰的に探索している)
// (isComplete()にてデータ構造が正しい事を確認してから実行する事)
void RIFF::setSize() {
	Data *data = RIFF::data[0];
	data->setSize(); // 包含データのsetSize()呼び出し
	size = data->getClassSize();
}

// クラス全体の大きさを取得する
//(実行前にsizeメンバが設定されていなければならない)
int RIFF::getClassSize() {
	return strlen(getIdentifierFromBase()) + sizeof(size) + size;
}
int RIFF::getSize() {
	return size;
}

Data* RIFF::getData(string identifier) {
	if(data.size() ==0) {
		return NULL;
	}
	if (identifier=="") {
		return data[0];
	} else {
		HARKIO_THROW_EXCEPTION("It's future function and not implemented yet.");
	}
}
vector<Data*> RIFF::getDatas() {
	return data;
}
void RIFF::addData(Data *data) {
	//! RIFFへDataを追加する前に、エラーチェックを行う
	addDataCheck(data);
	this->data.push_back(data);
}

/**********************************************************************
 HARK派生クラスメンバ関数
 **********************************************************************/

 // サイズ以外のデータが過不足なく設定されているかを調べる
// (データ構造を再帰的に探索している)
bool HARK::isComplete() {
	if(!data[0]) {
		HARKIO_SET_ERROR("Data object is null.");
		return false;
	}

	Data *data = RIFF::data[0];
	if(data->isComplete() != true) {
		return false;
	}
	return true;
}

void HARK::writeFile(string fullpath_file) {
	// データの整合性チェック
	if(isComplete() == false) {
		throw Error::getInstance();
	}
	// サイズフィールの確定
	setSize();

	try {
		// Open
		ofstream fs(fullpath_file.c_str(), ios::binary);
		if (fs.fail()) {
			HARKIO_THROW_ERROR(string("Can't open file :") + fullpath_file);
		}
		// 識別子
		fs.write((char*)getIdentifier(), strlen(getIdentifier()));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write HARK's data.");
		}
		// サイズ
		fs.write((char*)LToHPtr(&size), sizeof(size));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write HARK's data.");
		}

		// サイズの直後のシーク位置を取得
		int start_seek = fs.tellp();

		// Data の書き込み
		for(unsigned int i=0; i < data.size(); i++) {
			data[i]->writeFile(fs);
		}
		// 書き込み開始時のシーク位置と、サイズを比較
		// サイズチェックは、読み込み時に詳細に行っているので、書き込み時は全体のチェックしか行っていない。
		if ((int)fs.tellp() - start_seek != getSize()) {
			stringstream ss;
			ss << (int)fs.tellp() - start_seek << " Byte of " << getIdentifier() << " was written that header size is " << getSize() << " Byte.";
			HARKIO_THROW_ERROR(ss.str());
		}

		// Close
		fs.close();
	} catch(Error &err) {
		throw;
	}
	return;
}

void HARK::readFile(string fullpath_file) {
	try {
		char cha_buf[strlen(getIdentifier()) + 1];
		// Open
		ifstream fs(fullpath_file.c_str(), ios::binary);
		if (fs.fail()) {
			HARKIO_THROW_ERROR(string("Can't open file :") + fullpath_file);
		}

		// HGTF の新規作成
		Data *data = new HGTF(); // バージョンは入れなくてよい
		addData(data);
		HGTF *hgtf = dynamic_cast<HGTF*>(data);

		// 識別子
		memset(cha_buf, '\0', sizeof(cha_buf));
		fs.read((char*)cha_buf, strlen(getIdentifier()));
		if(fs.fail() || string(cha_buf) != getIdentifier()) {
			HARKIO_THROW_ERROR(string("RIFF's identifier was't ") + getIdentifier() + ".");
		}
		// サイズ
		fs.read((char*)&size, sizeof(int));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read RIFF's size.");
		}
		LToH(&size);

		// サイズの直後のシーク位置取得
		int start_seek = fs.tellg();

		// 包含データの読み込み関数呼び出し
		hgtf->readFile(fs);

		// 読み取り開始時のシーク位置と、サイズを比較
		if ((int)fs.tellg() - start_seek != getSize()) {
			stringstream ss;
			ss << (int)fs.tellg() - start_seek << " Byte of " << getIdentifier() << " was read that header size is " << getSize() << " Byte.";
			HARKIO_THROW_ERROR(ss.str());
		}

		fs.close();
	} catch(Error &e) {
		throw e;
	}
	return;
}

} // namespace
